#ifndef __SDTBARCODE_H__
#define __SDTBARCODE_H__

#ifdef __cplusplus
extern "C" {
#endif


// Supported Barcode Types Flags
#define SDTBARCODETYPE_UNKNOWN		0x00000000// Unknown
#define SDTBARCODETYPE_CODABAR		0x00000001// Codabar
#define SDTBARCODETYPE_CODE128		0x00000002// Code 128
#define SDTBARCODETYPE_CODE32		0x00000004// Code 32 (also known as Base 32, Pharma 32/39, and Italian Pharmacode)
#define SDTBARCODETYPE_CODE39		0x00000008// Code 39
#define SDTBARCODETYPE_CODE93		0x00000010// Code 93
#define SDTBARCODETYPE_CODE11		0x00000020// Code 11
#define SDTBARCODETYPE_EAN13		0x00000040// EAN13,
#define SDTBARCODETYPE_EAN8			0x00000080// EAN8
#define SDTBARCODETYPE_EAN5			0x00000100// EAN5
#define SDTBARCODETYPE_I2OF5		0x00000200// I2of5 (Interleaved 2 of 5)
#define SDTBARCODETYPE_PATCH_CODE	0x00000400// Patch code
#define SDTBARCODETYPE_POSTNET		0x00000800// Postnet
#define SDTBARCODETYPE_PLUS2		0x00001000// Plus2 (2-digit supplementals assoc. with EAN and UPC)
#define SDTBARCODETYPE_PLUS5		0x00002000// Plus5 (5-digit supplementals assoc. with EAN and UPC)
#define SDTBARCODETYPE_UPCA			0x00004000// UPC-A
#define SDTBARCODETYPE_UPCB			0x00008000// UPC-B
#define SDTBARCODETYPE_MSI			0x00100000// MSI
#define SDTBARCODETYPE_UPCE			0x00200000// UPC-E
#define SDTBARCODETYPE_PHARMACODE	0x00400000// PHARMACODE

#define SDTBARCODETYPE_ALL_1D		0x0070ffff// All above 1-D Barcodes

#define SDTBARCODETYPE_INTELLIMAIL	0x00010000// Intelligent Mail
#define SDTBARCODETYPE_DATAMATRIX	0x00020000// DataMatrix
#define SDTBARCODETYPE_PDF417		0x00040000// PDF417
#define SDTBARCODETYPE_QRCODE		0x00080000// QR Code

#define SDTBARCODETYPE_ALL_2D		0x000f0000// All above 2-D Barcodes

#define SDTINPUT_FILETYPE_BMP		1
#define SDTINPUT_FILETYPE_JPEG		2
#define SDTINPUT_FILETYPE_PNG		3
#define SDTINPUT_FILETYPE_TIFF		4
#define SDTINPUT_FILETYPE_PDF		5

#define SDTREADDIRECTION_LTR		0x01 // Left-to-right
#define SDTREADDIRECTION_RTL		0x02 // Right-to-Left
#define SDTREADDIRECTION_TTB		0x04 // Top-to-Bottom
#define SDTREADDIRECTION_BTT		0x08 // Bottom-to-top

#define SDTREADDIRECTION_ALL		0x0f // All directions

typedef void* LPSDBARREADER;

/// <summary>Create instance of SDT Barcode reader</summary>
/// <returns>If the function succeeds, the return value is a reader handle.
/// If the function fails, the return value is NULL. To get extended error information, check errno value.
/// </returns>
/// <param name="lpLicenseKey">[in] Purchased License Key</param>
LPSDBARREADER SDTCreateBarcodeReader(const wchar_t* lpLicenseKey);

/// <summary>Sets the License upgrade key. This key is used with developer key for access to newer version of runtimes.</summary>
/// <returns>If the function succeeds, the return value is a reader handle.
/// If the function fails, the return value is NULL. To get extended error information, check errno value.
/// </returns>
/// <param name="hReader">[in] Reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="lpLicenseUpgradeKey">[in] Purchased License Upgrade Key</param>
void SDTSetLicenseUpgradeKey(LPSDBARREADER hReader, const wchar_t* lpLicenseUpgradeKey);

/// <summary>Destroy previously created SDT Barcode reader instance</summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// </returns>
/// <param nam="hReader">[in] Reader handle previously created with CreateSDTBarcodeReader</param>
unsigned long SDTDestroyBarcodeReader(LPSDBARREADER hReader);

/// <summary>Define the square area within an image where the reader will look
///	for barcodes.
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// </returns>
/// <param nam="hReader">[in] Reader handle previously created with CreateSDTBarcodeReader</param>
/// <param nam="lprc">[in] Pointer to RECT structure with area values</param>
unsigned long SDTSetActiveScanRect(LPSDBARREADER hReader, int left, int top, int right, int bottom);

/// <summary>Define the barcode types the reader will look for. By default the reader look for all
/// barcode types. In order to achieve better performance it is recommended to minimize the set
/// of types by specifying only types really needed.
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// </returns>
/// <param nam="hReader">[in] Reader handle previously created with CreateSDTBarcodeReader</param>
/// <param nam="dwFlags">[in] Combination of one or more flags defined barcode type.
//	The following flags are supported by current version:
/// 1D or linear barcodes:
/// SDTBARCODETYPE_CODABAR - Codabar;
/// SDTBARCODETYPE_CODE128 - Code 128;
/// SDTBARCODETYPE_CODE32 - Code 32 also known as Base 32, Pharma 32/39 and Italian Pharmacode;
/// SDTBARCODETYPE_CODE39 - Code 39
/// SDTBARCODETYPE_CODE93 - Code 93
/// SDTBARCODETYPE_CODE11 - Code 11
/// SDTBARCODETYPE_EAN13 - EAN13,
/// SDTBARCODETYPE_EAN8 - EAN8
/// SDTBARCODETYPE_EAN5 - EAN5
/// SDTBARCODETYPE_I2OF5 - Interleaved 2 of 5
/// SDTBARCODETYPE_PATCH_CODE - Patch code
/// SDTBARCODETYPE_POSTNET - Postnet
/// SDTBARCODETYPE_PLUS2 - Plus2 (2-digit supplementals assoc. with EAN and UPC)
/// SDTBARCODETYPE_PLUS5 - Plus5 (5-digit supplementals assoc. with EAN and UPC)
/// SDTBARCODETYPE_UPCA - UPC-A
/// SDTBARCODETYPE_UPCB - UPC-B
/// SDTBARCODETYPE_ALL_1D - All above 1-D Barcodes
///
/// SDTBARCODETYPE_INTELLIMAIL - Intelligent Mail
/// SDTBARCODETYPE_DATAMATRIX - DataMatrix
/// SDTBARCODETYPE_PDF417 - PDF417
/// SDTBARCODETYPE_QRCODE - QR Code
/// SDTBARCODETYPE_ALL_2D - All above 2-D Barcodes
/// </param>
unsigned long SDTSetReadInputTypes(LPSDBARREADER hReader,
					unsigned long dwFlags);

/// <summary>Define read directions for linear barcodes.
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// </returns>
/// <param nam="hReader">[in] Reader handle previously created with CreateSDTBarcodeReader</param>
/// <param nam="dwFlags">[in] Combination of one or more flags defined direction.
//	The following flags are supported by current version:
/// SDTREADDIRECTION_LTR - Left-to-right
/// SDTREADDIRECTION_RTL - Right-to-Left
/// SDTREADDIRECTION_TTB - Top-to-Bottom
/// SDTREADDIRECTION_BTT - Bottom-to-top
/// SDTREADDIRECTION_ALL - All directions
/// </param>
unsigned long SDTSetReadInputDirections(LPSDBARREADER hReader,
						unsigned long dwFlags);

/// <summary>Read RGB image buffer and recognize barcode on it. Results are stored internally and 
/// can be obtained by calling SDTGetResultValueW or SDTGetResultValueA
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="lpImageBuffer">[in] pointer to allocated memory with RGB data.</param>
/// <param name="wImageWidth">[in] image width in pixels.</param>
/// <param name="wImageHeight">[in] image height in pixels.</param>
/// <param name="wBpp">[in] image buffer bits per pixel. Supported values are 32, 24, 16, 8.</param>
unsigned long SDTReadRGBImageBuffer(LPSDBARREADER hReader,
					unsigned char* lpImageBuffer,
					int wImageWidth,
					int wImageHeight,
					int wBpp);

/// <summary>Read image file and recognize barcode on it. Results are stored internally and 
/// can be obtained by calling SDTGetResultValueW or SDTGetResultValueA
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// When nonexisting page number is passed the function will return negative result.
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="lpFileName">[in] file name as ASCII string</param>
/// <param name="iPageIndex">[in] Page index of multipage image or 0. When nonexisting page number is passed
/// the function will return negative result</param>
unsigned long SDTReadImageFileA(LPSDBARREADER hReader, const char* lpFileName, int iPageIndex);

/// <summary>Read image file and recognize barcode on it. Results are stored internally and 
/// can be obtained by calling SDTGetResultValueW or SDTGetResultValueA
/// </summary>
/// <returns>If the function succeeds, the return value is zero.
/// If the function fails, the return value is nonzero and contain extended error information.
/// When nonexisting page number is passed the function will return negative result.
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="lpFileName">[in] file name as unicode string</param>
/// <param name="iPageIndex">[in] Page index of multipage image or 0. When nonexisting page number is passed
/// the function will return negative result</param>
unsigned long SDTReadImageFileW(LPSDBARREADER hReader, const wchar_t* lpFileName, int iPageIndex);

/// <summary>Return count of successfully recognized barcodes in the previous call 
/// to either SDTReadImageFileA, or SDTReadImageFileW, or SDTReadRGBImageBuffer. 
/// </summary>
/// <returns>Function returns count of barcodes or 0 in case of no barcodes were detected.</returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
int SDTGetResultsCount(LPSDBARREADER hReader);

/// <summary>The function returns ascii value of the successfully recognized barcode in the previous call 
/// to either SDTReadImageFileA, or SDTReadImageFileW, or SDTReadRGBImageBuffer
/// </summary>
/// <returns>Function returns ascii string containing the barcode value.</returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is 
/// obtained by call to SDTGetResultsCount
const char* SDTGetResultValueA(LPSDBARREADER hReader, int iPos);

/// <summary>The function returns unicode value of the successfully recognized barcode in the previous call 
/// to either SDTReadImageFileA, or SDTReadImageFileW, or SDTReadRGBImageBuffer
/// </summary>
/// <returns>Function returns unicode string containing the barcode value.</returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is 
/// obtained by call to SDTGetResultsCount
const wchar_t* SDTGetResultValueW(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode type name like EAN13.
/// </summary>
/// <returns>Pointer to unicode string which contains recognized barcode type name
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
const wchar_t* SDTGetResultTypeNameW(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode type name like EAN13.
/// </summary>
/// <returns>Pointer to ascii string which contains recognized barcode value
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
const char* SDTGetResultTypeNameA(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode type flag.
/// </summary>
/// <returns>One of SDTBARCODETYPE_ flags
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultType(LPSDBARREADER hReader, int iPos);


/// <summary>Get recognized barcode read direction.
/// </summary>
/// <returns>One of SDTREADDIRECTION_ flags
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultReadDirection(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode left boundary on the image.
/// </summary>
/// <returns>left position in pixels
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultPositionLeft(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode top boundary on the image.
/// </summary>
/// <returns>top position in pixels
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultPositionTop(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode right boundary on the image.
/// </summary>
/// <returns>right position in pixels
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultPositionRight(LPSDBARREADER hReader, int iPos);

/// <summary>Get recognized barcode bottom boundary on the image.
/// </summary>
/// <returns>bottom position in pixels
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultPositionBottom(LPSDBARREADER hReader, int iPos);

/// <summary>Get size of byte buffer containing recognized barcode value as byte array
/// </summary>
/// <returns>Size of byte buffer which contains recognized barcode value
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned long SDTGetResultValueAsBinaryBufferSize(LPSDBARREADER hReader, int iPos);

/// <summary>Get pointer to byte buffer containing recognized barcode value as byte array
/// </summary>
/// <returns>Pointer to buffer which contains recognized barcode value
/// </returns>
/// <param name="hReader">[in] reader handle previously created with CreateSDTBarcodeReader</param>
/// <param name="iPos">[in] index of recognized barcode result. The count of available barcode results is obtained by call to SDTGetResultsCount() </param>
unsigned char* SDTGetResultValueAsBinaryBuffer(LPSDBARREADER hReader, int iPos);

#ifdef __cplusplus
}
#endif


#endif //__SDTBARCODE_H__
